-- Made By Frugle. Refactored by Zivres.

local acceptablePercentageSubjectsByChatCommand = {player={"player"}, target={"target"}, focus={"focus"}, boss={"boss1", "boss2", "boss3", "boss4"}}
local acceptableFontSubjectsByChatCommand = {player={"player"}, target={"target"}, focus={"focus"}, boss={"boss1", "boss2", "boss3", "boss4"}, all={"player", "target", "focus", "boss1", "boss2", "boss3", "boss4"}}
local subjectDisplayNameByChatCommand = {player="Player", target="Target", focus="Focus", boss="Boss additional frame", all="All"}

local Frames = {}
local OriginalFrames = {player=PlayerFrame, target=TargetFrame, focus=FocusFrame, boss1=Boss1TargetFrame, boss2=Boss2TargetFrame, boss3=Boss3TargetFrame, boss4=Boss4TargetFrame}


local addonLoadedFrame = CreateFrame("FRAME", "addonLoadedFrame")
addonLoadedFrame:RegisterEvent("ADDON_LOADED")
addonLoadedFrame:SetScript("OnEvent", function (self, event, ...)
	Init(...)
end)

local playerLoggedInFrame = CreateFrame("FRAME", "playerLoggedInFrame")
playerLoggedInFrame:RegisterEvent("PLAYER_LOGIN")
playerLoggedInFrame:SetScript("OnEvent", function (self, event, ...)
	for k,v in pairs(OriginalFrames) do UpdateResources(k) end
end)

local hpHandlerFrame = CreateFrame("FRAME", "hpHandlerFrame")
hpHandlerFrame:RegisterEvent("UNIT_HEALTH_FREQUENT")
hpHandlerFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResource(..., "health")
end)

local powerHandlerFrame = CreateFrame("FRAME", "powerHandlerFrame")
powerHandlerFrame:RegisterEvent("UNIT_POWER_UPDATE")
powerHandlerFrame:RegisterEvent("UNIT_DISPLAYPOWER")
powerHandlerFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResource(..., "mana")
end)

local targetChangeFrame = CreateFrame("FRAME", "targetChangeFrame")
targetChangeFrame:RegisterEvent("PLAYER_TARGET_CHANGED")
targetChangeFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResources("target")
end)


local focusChangeFrame = CreateFrame("FRAME", "focusChangeFrame")
focusChangeFrame:RegisterEvent("PLAYER_FOCUS_CHANGED")
focusChangeFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResources("focus")
end)

local bossFightStartFrame = CreateFrame("FRAME", "bossFightStartFrame")
bossFightStartFrame:RegisterEvent("INSTANCE_ENCOUNTER_ENGAGE_UNIT")
bossFightStartFrame:SetScript("OnEvent", function (self, event, ...)
	UpdateResources("boss1")
	UpdateResources("boss2")
	UpdateResources("boss3")
	UpdateResources("boss4")
end)





function Init(addon)
	if addon == "ANewSimplePercent" then
		if SimplePercent == nil then
			SimplePercent = {}
		end
		for k,v in pairs(OriginalFrames) do InitFrames(k) end
	end
end

function InitFrames(unitId)
	local path, fontSize, flags = InitFont(unitId)
	InitFrame(unitId, "health", path, fontSize, flags)
	InitFrame(unitId, "mana", path, fontSize, flags)
end

function InitFont(unitId)
	local path, fontSize, flags = PlayerFrameHealthBarText:GetFont()
	if SimplePercent[unitId .. "FontSize"] == nil then
		SimplePercent[unitId .. "FontSize"] = fontSize
	end
	return path, SimplePercent[unitId .. "FontSize"], flags
end

function InitFrame(unitId, type, path, fontSize, flags)
	local setting = unitId .. type
	if SimplePercent[setting] == nil then
		SimplePercent[setting] = true
	end
	local frame = OriginalFrames[unitId]:CreateFontString(setting .. "Frame", "OVERLAY")
	Frames[setting] = frame
	frame:SetFont(path, fontSize, flags)
	frame:SetJustifyV("TOP")

	if unitId == "player" then
		frame:SetJustifyH("RIGHT")
	else
		frame:SetJustifyH("LEFT")
	end
	if type == "health" then
		frame.PointYos = 0
	else
		frame.PointYos = -3
	end
	SetFrameState(frame, SimplePercent[setting])
end

function UpdateResources(unitId)
	UpdateResource(unitId, "health")
	UpdateResource(unitId, "mana")
end

function UpdateResource(unitId, type)
	if OriginalFrames[unitId] == nil then
		return
	end

	local frameId = unitId .. type

	if SimplePercent[frameId] == false then
		return
	end

	local resource = UnitResourse(unitId, type)
	local maxResource = UnitResourseMax(unitId, type)
	
	local percentage
	if resource == 0 or UnitIsDeadOrGhost(unitId) then
		percentage = 0
	elseif resource == maxResource then
		percentage = 100
	elseif maxResource == 100 then
		percentage = resource
	else
		percentage = math.floor(resource / maxResource * 100)
	end

	local frame = Frames[frameId]
	if frame.percentage == percentage then
		return
	end

	frame.percentage = percentage
	local percentageString = tostring(percentage) .. "%"
	frame:SetText(percentageString)

	refreshPanelsPosition(unitId, type, frame)
end

function UnitResourse(unitId, type)
	if type == "health" then
		return UnitHealth(unitId)
	end
	return UnitPower(unitId)
end

function UnitResourseMax(unitId, type)
	if type == "health" then
		return UnitHealthMax(unitId)
	end
	return UnitPowerMax(unitId)
end

function SetFrameState(frame, isVisible)
	if isVisible == false then
		frame:Hide()
	else
		frame:Show()
	end
end


SLASH_CHANGE_COMMAND1 = "/simplepercent"
SlashCmdList["CHANGE_COMMAND"] = function (msg, editbox)
	local splitted = {}
	for word in msg:gmatch("[^%s]+") do 
    		table.insert(splitted, word)
	end
	if getn(splitted) == 2 then
		local subjects = acceptablePercentageSubjectsByChatCommand[splitted[1]]
		if subjects ~= nil and (splitted[2] == "health" or splitted[2] == "mana") then
			for k,v in pairs(subjects) do HandleCommand(v, splitted[2]) end
			PrintReactionMessage(subjects[1], splitted[2], subjectDisplayNameByChatCommand[splitted[1]])
			return
		end
	elseif getn(splitted) == 3 then
		local subjects = acceptableFontSubjectsByChatCommand[splitted[1]]
		if subjects ~= nil and (splitted[2] == "font") then
			local size = tonumber(splitted[3]) 
			if size == nil and splitted[3] == "default" then
				size = splitted[3]
			end
			if size ~= nil and (size == "default" or size > 0) then
				for k,v in pairs(subjects) do HandleFontSize(v, size) end
				PrintFontReactionMessage(size, subjectDisplayNameByChatCommand[splitted[1]])
				return
			end
		end
	end
	print("You can toggle percent visiblity with this command")
	print("Syntax: /simplepercent <player or target or focus or boss> <health or mana> | Example: /simplepercent target health")
	print("You can also set font size manually")
	print("Syntax: /simplepercent <player or target or focus or boss or all> font <default or number> | Example: /simplepercent target font 11")
end


function HandleCommand(unitId, type)
	local frameId = unitId .. type

	if SimplePercent[frameId] == false then
		SimplePercent[frameId] = true
	else
		SimplePercent[frameId] = false
	end
	SetFrameState(Frames[frameId], SimplePercent[frameId])
	UpdateResource(unitId, type)
end


function PrintReactionMessage(unitId, type, unitDisplayName)
	local frameId = unitId .. type

	if SimplePercent[frameId] == true then
		print(unitDisplayName .. " " .. type .. " percent is now visible.")
	else
		print(unitDisplayName .. " " .. type .. " percent is now hidden.")
	end
end

function HandleFontSize(unitId, size)
	local propertyId = unitId .. "FontSize"
	if size == "default" then
		local Path, FontSize, Flags = PlayerFrameHealthBarText:GetFontObject():GetFont()
		size = FontSize
	end
	SimplePercent[propertyId] = size 

	refreshFont(unitId, "health", size)
	refreshFont(unitId, "mana", size)
end

function refreshFont(unitId, type, size)
	local frame = Frames[unitId .. type]
	local path, fontSize, flags = frame:GetFont()
	frame:SetFont(path, size, flags)
	refreshPanelsPosition(unitId, type, frame)
end

function PrintFontReactionMessage(size, unitDisplayName)
	print(unitDisplayName .. " percentage font size is now " .. size .. ".")
end

function refreshPanelsPosition(unitId, type, frame)
	if unitId == "player" then
		frame.PointXos = 0
		frame:SetPoint(getPercentagePoint(type), OriginalFrames[unitId], "RIGHT", frame.PointXos, frame.PointYos)
	else
		frame.PointXos = frame:GetStringWidth(percentageString) * -1
		frame:SetPoint(getPercentagePoint(type), OriginalFrames[unitId], "LEFT", frame.PointXos, frame.PointYos)
	end
end

function getPercentagePoint(type)
	if type == "health" then
		return "BOTTOMLEFT"
	else
		return "TOPLEFT"
	end
end